#!/bin/bash

# Note that this script stops rsyslog and renames /var/log temporarily. It won't clean up if there's
# an error further down the script, so bear that in mind.

# Run logwatch on two crafted log files: logwatch/messages and logwatch/yum.log. Three files are
# generated in the current working directory: d1.out, d2.out and d3.out. See comments below.
# As a reference, see contents of logwatch/fail/*.out and logwatch/pass/*.out for cases when
# the bug reproduces or is fixed respectively. Note that they are just a reference, the result
# won't likely be the same on another machine, as the output files contain a timestamped header
# with a hostname in it.


# Prepare. Note that the reproducer probably won't work if there's populated /var/log on the system,
# logwatch uses --logdir= as a place to look for the logs *in addition* to /var/log.
systemctl stop rsyslog
mv /var/log{,.old}

# Run logwatch and save the output.
logwatch --service=yum --service=arpwatch --range=all --detail=high --encode=base64 --logdir=./logwatch > /tmp/base64.out

# Coreutils base64 handles the invalid output just fine.
base64 -d /tmp/base64.out > d1.out

# Python 2.7.5 handles only the first block (until the first newline character).
python << EOF > d2.out
import base64
import sys
data = open('/tmp/base64.out', 'r').read()
sys.stdout.write(base64.b64decode(data))
EOF

# OpenSSL base64 outputs nothing if the output is invalid.
openssl base64 -d < /tmp/base64.out > d3.out

# Clean up.
mv /var/log{.old,}
systemctl start rsyslog
